package org.colinvella.fancyfish.world.gen.feature;

import java.util.Random;

import org.colinvella.fancyfish.block.ModBlockRegistry;
import org.colinvella.fancyfish.configuration.ModConfiguration;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.init.Blocks;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;
import net.minecraft.world.chunk.IChunkProvider;

public class KelpWorldGenerator extends ModWorldGenerator
{
	public static String ID = "Kelp";
	
    public KelpWorldGenerator()
    {
    	super(ID);
        this.kelpBlock = ModBlockRegistry.Kelp;
    }
    
	@Override
	public void generate(Random random, int chunkX, int chunkZ, World world, IChunkProvider chunkGenerator,
			IChunkProvider chunkProvider) 
	{
		// do not generate features if disabled in mod config
        if (!ModConfiguration.getTerrainGeneration())
        	return;
		
		// only works for regular overworld
		if (world.provider.getDimensionId() != 0)
			return;
		
		int patchesPerChunk = 1 + random.nextInt(3);
		for (int patchIndex = 0; patchIndex < patchesPerChunk; patchIndex++)
		{
			int genX = chunkX * 16 + random.nextInt(16);
			int genZ = chunkZ * 16 + random.nextInt(16);
			generateKelpPatch(world, random, genX, genZ);
		}
	}
	
	private void generateKelpPatch(World world, Random random, int genX, int genZ)
	{
		logger.debug("Generating kelp patch...");
		int kelpPatchRadius = 1 + random.nextInt(3);
		int kelpPatchRadiusSquared = kelpPatchRadius * kelpPatchRadius;
        int posY = world.getHeight() - 1;
        for (int posX = genX - kelpPatchRadius; posX <= genX + kelpPatchRadius; ++posX)
        {
            int deltaX = posX - genX;
            int deltaXSquared = deltaX * deltaX;
            for (int posZ = genZ - kelpPatchRadius; posZ <= genZ + kelpPatchRadius; ++posZ)
            {
                int deltaZ = posZ - genZ;

                if (deltaXSquared + deltaZ * deltaZ > kelpPatchRadiusSquared)
                	continue;
                
                posY += 32;
                BlockPos pos = new BlockPos(posX, posY, posZ);
                while(pos.getY() > 0)
                {
                	Block scanBlock = world.getBlockState(pos).getBlock();
                	if (scanBlock != Blocks.air && scanBlock != Blocks.water)
                		break;
                	pos = pos.down();
                }
                
                Block baseBlock = world.getBlockState(pos).getBlock();
                if (baseBlock != Blocks.dirt && baseBlock != Blocks.sand && baseBlock != Blocks.clay && baseBlock != kelpBlock)
                	continue;

                int kelpHeight = 1 + random.nextInt(3);

                while (kelpHeight-- > 0)
                {
                	pos = pos.up();
                	
                	if (world.getBlockState(pos).getBlock() != Blocks.water)
                		break;

                	if (world.getBlockState(pos.up()).getBlock() == Blocks.air)
                		break;

                	world.setBlockState(pos, this.kelpBlock.getDefaultState());
                }
            }
        }
		
	}
	
    private Block kelpBlock;
}
